﻿using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Http;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace Using_Viewer_Events
{
    public class AuthorisationKeyHandler : AuthorizationHandler<AuthorisationKeyRequirement>
    {
        /// <summary>
        /// Authorisation key header name.
        /// </summary>
        public const string ApiKeyHeaderName = "ApiKey";

        private readonly IHttpContextAccessor httpContextAccessor;

        /// <summary>
        /// Initializes a new instance of the <see cref="AuthorisationKeyHandler"/> class.
        /// </summary>
        /// <param name="httpContextAccessor">The Http Context Accessor.</param>
        public AuthorisationKeyHandler(IHttpContextAccessor httpContextAccessor)
        {
            this.httpContextAccessor = httpContextAccessor;
        }

        /// <inheritdoc/>
        protected override Task HandleRequirementAsync(AuthorizationHandlerContext context, AuthorisationKeyRequirement requirement)
        {
            this.SucceedRequirementIfAuthKeyPresentAndValid(context, requirement);
            return Task.CompletedTask;
        }

        private void SucceedRequirementIfAuthKeyPresentAndValid(AuthorizationHandlerContext context, AuthorisationKeyRequirement requirement)
        {
            var authKey = this.httpContextAccessor.HttpContext.Request.Headers[ApiKeyHeaderName].FirstOrDefault();
            if (authKey != null && requirement.AuthorisationKeys.Any(requiredAuthKey => authKey == requiredAuthKey))
            {
                context.Succeed(requirement);
            }
        }
    }
}
